/*
  Copyright 2019 Kakao Corp.
  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at
     http://www.apache.org/licenses/LICENSE-2.0
  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.sample.common

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.android.material.snackbar.BaseTransientBottomBar
import com.google.android.material.snackbar.Snackbar
import com.kakao.sdk.auth.network.RxAuthOperations
import com.kakao.sdk.common.model.ClientError
import com.kakao.sdk.common.model.ClientErrorCause
import com.kakao.sdk.common.model.KakaoSdkError
import com.kakao.sdk.friend.client.rx
import com.kakao.sdk.friend.core.PickerClient
import com.kakao.sdk.friend.core.model.DisableSelectOption
import com.kakao.sdk.friend.core.model.DisableSelectReason
import com.kakao.sdk.friend.core.model.PickerChatFilter
import com.kakao.sdk.friend.core.model.PickerChatSelectionType
import com.kakao.sdk.friend.core.model.PickerFriendFilter
import com.kakao.sdk.friend.core.model.PickerOrientation
import com.kakao.sdk.friend.core.model.PickerUsingOsFilter
import com.kakao.sdk.friend.core.model.ViewAppearance
import com.kakao.sdk.partner.friend.client.select
import com.kakao.sdk.partner.friend.client.selectChat
import com.kakao.sdk.partner.friend.client.selectChatPopup
import com.kakao.sdk.partner.friend.client.selectFriend
import com.kakao.sdk.partner.friend.client.selectFriendPopup
import com.kakao.sdk.partner.friend.client.selectFriends
import com.kakao.sdk.partner.friend.client.selectFriendsPopup
import com.kakao.sdk.partner.friend.client.selectPopup
import com.kakao.sdk.partner.friend.model.PickerChatRequestParams
import com.kakao.sdk.partner.friend.model.PickerFriendRequestParams
import com.kakao.sdk.partner.friend.model.PickerTabRequestParams
import com.kakao.sdk.partner.sample.common.internal.fromChat
import com.kakao.sdk.partner.sample.common.internal.fromPartnerFriend
import com.kakao.sdk.partner.talk.chatList
import com.kakao.sdk.partner.talk.chatMembers
import com.kakao.sdk.partner.talk.friendsForPartner
import com.kakao.sdk.partner.talk.sendCustomMessageForPartner
import com.kakao.sdk.partner.talk.sendCustomMessageToChats
import com.kakao.sdk.partner.talk.sendDefaultMessageForPartner
import com.kakao.sdk.partner.talk.sendDefaultMessageToChats
import com.kakao.sdk.partner.talk.sendScrapMessageForPartner
import com.kakao.sdk.partner.talk.sendScrapMessageToChats
import com.kakao.sdk.partner.user.ageAuthInfo
import com.kakao.sdk.partner.user.createShippingAddress
import com.kakao.sdk.partner.user.meForPartner
import com.kakao.sdk.partner.user.model.AgeAuthLevel
import com.kakao.sdk.partner.user.signupForPartner
import com.kakao.sdk.partner.user.updateShippingAddress
import com.kakao.sdk.partner.user.upgradeScopes
import com.kakao.sdk.partner.user.verifyAge
import com.kakao.sdk.sample.common.OpenFragmentRx
import com.kakao.sdk.sample.common.internal.ApiAdapter
import com.kakao.sdk.sample.common.internal.FriendsActivity
import com.kakao.sdk.sample.common.internal.Log
import com.kakao.sdk.sample.common.internal.PickerItem
import com.kakao.sdk.talk.TalkApiClient
import com.kakao.sdk.talk.rx
import com.kakao.sdk.user.UserApiClient
import com.kakao.sdk.user.rx
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.core.Single
import io.reactivex.rxjava3.disposables.CompositeDisposable
import io.reactivex.rxjava3.functions.Consumer
import io.reactivex.rxjava3.kotlin.addTo
import io.reactivex.rxjava3.schedulers.Schedulers

/**
 * A simple [Fragment] subclass.
 */
class PartnerFragmentRx : Fragment() {

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.let { templateIds = it }
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_partner, container, false)
    }

    private val onSuccess = Consumer<Any> {
        showSnackbar(it)
    }

    private val onError = Consumer<Throwable> {
        showSnackbar(it)
    }

    private fun showSnackbar(any: Any) {
        val snackbar =
            Snackbar.make(requireView(), any.toString(), BaseTransientBottomBar.LENGTH_LONG)
        val layout = snackbar.view as Snackbar.SnackbarLayout
        layout.minimumHeight = 100
        snackbar.show()
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        viewManager = LinearLayoutManager(requireContext())
        viewAdapter = ApiAdapter(
            listOf(
                ApiAdapter.Item.Header("Auth API"),
                ApiAdapter.Item.ApiItem("verifyAge(default) ") {
                    UserApiClient.rx.verifyAge(requireContext())
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe({
                            Log.i(TAG, "연령 인증 성공")
                            showSnackbar("연령 인증 성공")

                        }, { error ->
                            Log.e(TAG, "연령 인증 실패", error)
                            showSnackbar("연령 인증 실패")
                        }).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("verifyAge(1차 인증) ") {
                    UserApiClient.rx.verifyAge(
                        requireContext(),
                        authLevel = AgeAuthLevel.AUTH_LEVEL1
                    )
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe({
                            Log.i(TAG, "연령 인증 성공")
                            showSnackbar("연령 인증 성공")
                        }, { error ->
                            Log.e(TAG, "연령 인증 실패", error)
                            showSnackbar("연령 인증 실패")
                        }).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("verifyAge(2차 인증) ") {
                    UserApiClient.rx.verifyAge(
                        requireContext(),
                        authLevel = AgeAuthLevel.AUTH_LEVEL2
                    )
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe({
                            Log.i(TAG, "연령 인증 성공")
                            showSnackbar("연령 인증 성공")
                        }, { error ->
                            Log.e(TAG, "연령 인증 실패", error)
                            showSnackbar("연령 인증 실패")
                        }).addTo(disposables)
                },
                ApiAdapter.Item.Header("User API"),
                ApiAdapter.Item.ApiItem("loginWithKakaoAccount(accountParameters:) - empty") {
                    UserApiClient.rx.loginWithKakaoAccount(requireContext())
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe({ token ->
                            Log.i(TAG, "로그인 성공 ${token.accessToken}")
                        }, { error ->
                            Log.e(TAG, "로그인 실패", error)
                        }).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("loginWithKakaoAccount(accountParameters:) - login '@kakaocorp.com'") {
                    UserApiClient.rx.loginWithKakaoAccount(
                        requireContext()
                    )
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe({ token ->
                            Log.i(TAG, "로그인 성공 ${token.accessToken}")
                        }, { error ->
                            Log.e(TAG, "로그인 실패", error)
                        })
                        .addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("loginWithKakaoAccount(accountParameters:) - signup") {
                    UserApiClient.rx.loginWithKakaoAccount(
                        requireContext()
                    )
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe({ token ->
                            Log.i(TAG, "로그인 성공 ${token.accessToken}")
                        }, { error ->
                            Log.e(TAG, "로그인 실패", error)
                        }).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("meForPartner()") {
                    UserApiClient.rx.meForPartner()
                        .subscribeOn(Schedulers.io())
                        .subscribe(onSuccess, onError).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("signupForPartner()") {
                    UserApiClient.rx.signupForPartner()
                        .subscribeOn(Schedulers.io())
                        .subscribe({ showSnackbar("signupForPartner() 성공") }, onError)
                        .addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("ageAuthInfo()") {
                    UserApiClient.rx.ageAuthInfo()
                        .subscribeOn(Schedulers.io())
                        .subscribe(onSuccess, onError).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("upgradeScopes()") {
                    val scopes = listOf("birthday")
                    UserApiClient.rx.upgradeScopes(scopes)
                        .subscribeOn(Schedulers.io())
                        .subscribe(onSuccess, onError).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("createShippingAddress() ") {
                    UserApiClient.rx.createShippingAddress(requireContext())
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe({ addressId ->
                            Log.i(TAG, "배송지 추가 성공 $addressId")
                            showSnackbar("배송지 추가 성공 $addressId")
                        }, { error ->
                            Log.i(TAG, "배송지 추가 실패 $error")
                            showSnackbar("배송지 추가 실패 $error")
                        })
                        .addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("updateShippingAddress() ") {
                    UserApiClient.rx.selectShippingAddress(requireContext())
                        .observeOn(AndroidSchedulers.mainThread())
                        .flatMap { addressId ->
                            UserApiClient.rx.updateShippingAddress(requireContext(), addressId)
                        }
                        .subscribe({ addressId ->
                            Log.i(TAG, "배송지 수정 성공 $addressId")
                            showSnackbar("배송지 수정 성공 $addressId")
                        }, { error ->
                            Log.i(TAG, "배송지 수정 실패 $error")
                            showSnackbar("배송지 수정 실패 $error")
                        })
                        .addTo(disposables)
                },
                ApiAdapter.Item.Header("Friend Picker API"),
                ApiAdapter.Item.ApiItem("selectFriend") {
                    val pickerFriendRequestParams = PickerFriendRequestParams(
                        title = "샘플앱 싱글 친구피커", //default "카카오톡 친구 선택"
                        friendFilter = null, //default PickerFriendFilter.NONE
                        countryCodeFilters = null, //default null
                        usingOsFilter = PickerUsingOsFilter.ALL, //default PickerUsingOsFilter.ALL
                        viewAppearance = ViewAppearance.AUTO, //default ViewAppearance.AUTO
                        orientation = PickerOrientation.AUTO, //default PickerOrientation.AUTO
                        enableSearch = true, //default true
                        enableIndex = true, //default true
                        showMyProfile = true, //default true
                        showFavorite = true, //default true
                        disableSelectOptions = listOf(DisableSelectOption(DisableSelectReason.MSG_BLOCKED)), // default null
                        displayAllProfile = null //default false 미동의 사용자 프로필 응답 권한 필요
                    )

                    PickerClient.rx.selectFriend(
                        context = requireContext(),
                        params = pickerFriendRequestParams
                    ).subscribe({ selectedUsers ->
                        Log.d(TAG, "친구 선택 성공 $selectedUsers")
                        showSnackbar("친구 선택 성공 $selectedUsers")
                    }, { error ->
                        Log.e(TAG, "친구 선택 실패", error)
                        showSnackbar("친구 선택 실패 $error")
                    }).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("selectFriendPopup") {
                    val pickerFriendRequestParams = PickerFriendRequestParams(
                        title = "샘플앱 싱글 친구피커", //default "카카오톡 친구 선택"
                        friendFilter = null, //default PickerFriendFilter.NONE
                        countryCodeFilters = null, //default null
                        usingOsFilter = PickerUsingOsFilter.ALL, //default PickerUsingOsFilter.ALL
                        viewAppearance = ViewAppearance.AUTO, //default ViewAppearance.AUTO
                        orientation = PickerOrientation.AUTO, //default PickerOrientation.AUTO
                        enableSearch = true, //default true
                        enableIndex = true, //default true
                        showMyProfile = true, //default true
                        showFavorite = true, //default true
                        disableSelectOptions = listOf(DisableSelectOption(DisableSelectReason.MSG_BLOCKED)), // default null
                        displayAllProfile = null //default false 미동의 사용자 프로필 응답 권한 필요
                    )

                    PickerClient.rx.selectFriendPopup(
                        context = requireContext(),
                        params = pickerFriendRequestParams
                    ).subscribe({ selectedUsers ->
                        Log.d(TAG, "친구 선택 성공 $selectedUsers")
                        showSnackbar("친구 선택 성공 $selectedUsers")
                    }, { error ->
                        Log.e(TAG, "친구 선택 실패", error)
                        showSnackbar("친구 선택 실패 $error")
                    }).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("selectFriends") {
                    val pickerFriendRequestParams = PickerFriendRequestParams(
                        title = "샘플앱 멀티 친구피커", //default "카카오톡 친구 선택"
                        friendFilter = null, //default PickerFriendFilter.NONE
                        countryCodeFilters = null, //default null
                        usingOsFilter = PickerUsingOsFilter.ALL, //default PickerUsingOsFilter.ALL
                        viewAppearance = ViewAppearance.AUTO, //default ViewAppearance.AUTO
                        orientation = PickerOrientation.AUTO, //default PickerOrientation.AUTO
                        enableSearch = true, //default true
                        enableIndex = true, //default true
                        showMyProfile = true, //default true
                        showFavorite = true, //default true
                        disableSelectOptions = listOf(DisableSelectOption(DisableSelectReason.MSG_BLOCKED)), // default null
                        displayAllProfile = null, //default false 미동의 사용자 프로필 응답 권한 필요
                        showPickedFriend = null, // default true
                        maxPickableCount = null, // default 30
                        minPickableCount = null // default 1
                    )

                    PickerClient.rx.selectFriends(
                        context = requireContext(),
                        params = pickerFriendRequestParams
                    ).subscribe({ selectedUsers ->
                        Log.d(TAG, "친구 선택 성공 $selectedUsers")
                        showSnackbar("친구 선택 성공 $selectedUsers")
                    }, { error ->
                        Log.e(TAG, "친구 선택 실패", error)
                        showSnackbar("친구 선택 실패 $error")
                    }).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("selectFriendsPopup") {
                    val pickerFriendRequestParams = PickerFriendRequestParams(
                        title = "샘플앱 멀티 친구피커", //default "카카오톡 친구 선택"
                        friendFilter = null, //default PickerFriendFilter.NONE
                        countryCodeFilters = null, //default null
                        usingOsFilter = PickerUsingOsFilter.ALL, //default PickerUsingOsFilter.ALL
                        viewAppearance = ViewAppearance.AUTO, //default ViewAppearance.AUTO
                        orientation = PickerOrientation.AUTO, //default PickerOrientation.AUTO
                        enableSearch = true, //default true
                        enableIndex = true, //default true
                        showMyProfile = true, //default true
                        showFavorite = true, //default true
                        disableSelectOptions = listOf(DisableSelectOption(DisableSelectReason.MSG_BLOCKED)), // default null
                        displayAllProfile = null, //default false 미동의 사용자 프로필 응답 권한 필요
                        showPickedFriend = null, // default true
                        maxPickableCount = null, // default 30
                        minPickableCount = null // default 1
                    )

                    PickerClient.rx.selectFriendsPopup(
                        context = requireContext(),
                        params = pickerFriendRequestParams
                    ).subscribe({ selectedUsers ->
                        Log.d(TAG, "친구 선택 성공 $selectedUsers")
                        showSnackbar("친구 선택 성공 $selectedUsers")
                    }, { error ->
                        Log.e(TAG, "친구 선택 실패", error)
                        showSnackbar("친구 선택 실패 $error")
                    }).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("selectChat - CHAT") {
                    val pickerChatRequestParams = PickerChatRequestParams(
                        selectionType = PickerChatSelectionType.CHAT, // PickerChatSelectionType.CHAT 또는 PickerChatSelectionType.CHAT_MEMBER 선택 (필수)
                        title = "샘플앱 채팅방 피커", // default "채팅방 피커"
                        chatFilters = listOf(PickerChatFilter.REGULAR), // default listOf(PickerChatFilter.REGULAR)
                        viewAppearance = ViewAppearance.AUTO, // default ViewAppearance.AUTO
                        orientation = PickerOrientation.AUTO, // default PickerOrientation.AUTO
                        enableSearch = true // default true
                    )

                    PickerClient.rx.selectChat(requireContext(), pickerChatRequestParams)
                        .subscribe({ (_, selectedChat) ->
                            Log.d(TAG, "채팅방 선택 성공 $selectedChat")
                            showSnackbar("채팅방 선택 성공 $selectedChat")
                        }, { error ->
                            Log.e(TAG, "채팅방 선택 실패", error)
                            showSnackbar("채팅방 선택 실패 $error")
                        }).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("selectChatPopup - CHAT") {
                    val pickerChatRequestParams = PickerChatRequestParams(
                        selectionType = PickerChatSelectionType.CHAT, // PickerChatSelectionType.CHAT 또는 PickerChatSelectionType.CHAT_MEMBER 선택 (필수)
                        title = "샘플앱 채팅방 피커", // default "채팅방 피커"
                        chatFilters = listOf(PickerChatFilter.REGULAR), // default listOf(PickerChatFilter.REGULAR)
                        viewAppearance = ViewAppearance.AUTO, // default ViewAppearance.AUTO
                        orientation = PickerOrientation.AUTO, // default PickerOrientation.AUTO
                        enableSearch = true // default true
                    )

                    PickerClient.rx.selectChatPopup(requireContext(), pickerChatRequestParams)
                        .subscribe({ (_, selectedChat) ->
                            Log.d(TAG, "채팅방 선택 성공 $selectedChat")
                            showSnackbar("채팅방 선택 성공 $selectedChat")
                        }, { error ->
                            Log.e(TAG, "채팅방 선택 실패", error)
                            showSnackbar("채팅방 선택 실패 $error")
                        }).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("selectChat - CHAT_MEMBER") {
                    val pickerChatRequestParams = PickerChatRequestParams(
                        selectionType = PickerChatSelectionType.CHAT_MEMBER, // PickerChatSelectionType.CHAT 또는 PickerChatSelectionType.CHAT_MEMBER 선택 (필수)
                        title = "샘플앱 채팅방 멤버 피커", // default "채팅방 피커",
                        chatFilters = listOf(PickerChatFilter.REGULAR), // default listOf(PickerChatFilter.REGULAR),
                        viewAppearance = ViewAppearance.AUTO, // default ViewAppearance.AUTO
                        orientation = PickerOrientation.AUTO, // default PickerOrientation.AUTO
                        enableSearch = true, // default true
                        disableSelectOptions = listOf(DisableSelectOption(DisableSelectReason.MSG_BLOCKED)), // default null
                        displayAllProfile = null, // default false
                        maxPickableCount = null, // default 30
                        minPickableCount = null // default 1
                    )

                    PickerClient.rx.selectChat(requireContext(), pickerChatRequestParams)
                        .subscribe({ (selectedUsers, _) ->
                            Log.d(TAG, "채팅방 멤버 선택 성공 $selectedUsers")
                            showSnackbar("채팅방 멤버 선택 성공 $selectedUsers")
                        }, { error ->
                            Log.e(TAG, "채팅방 멤버 선택 실패", error)
                            showSnackbar("채팅방 멤버 선택 실패 $error")
                        }).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("selectChatPopup - CHAT_MEMBER") {
                    val pickerChatRequestParams = PickerChatRequestParams(
                        selectionType = PickerChatSelectionType.CHAT_MEMBER, // PickerChatSelectionType.CHAT 또는 PickerChatSelectionType.CHAT_MEMBER 선택 (필수)
                        title = "샘플앱 채팅방 멤버 피커", // default "채팅방 피커",
                        chatFilters = listOf(PickerChatFilter.REGULAR), // default listOf(PickerChatFilter.REGULAR),
                        viewAppearance = ViewAppearance.AUTO, // default ViewAppearance.AUTO
                        orientation = PickerOrientation.AUTO, // default PickerOrientation.AUTO
                        enableSearch = true, // default true
                        disableSelectOptions = listOf(DisableSelectOption(DisableSelectReason.MSG_BLOCKED)), // default null
                        displayAllProfile = null, // default false
                        maxPickableCount = null, // default 30
                        minPickableCount = null // default 1
                    )

                    PickerClient.rx.selectChatPopup(requireContext(), pickerChatRequestParams)
                        .subscribe({ (selectedUsers, _) ->
                            Log.d(TAG, "채팅방 멤버 선택 성공 $selectedUsers")
                            showSnackbar("채팅방 멤버 선택 성공 $selectedUsers")
                        }, { error ->
                            Log.e(TAG, "채팅방 멤버 선택 실패", error)
                            showSnackbar("채팅방 멤버 선택 실패 $error")
                        }).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("select - CHAT") {
                    val friendParams = PickerTabRequestParams.FriendsRequestParams(
                        friendFilter = PickerFriendFilter.NONE, // default PickerFriendFilter.NONE
                        countryCodeFilters = listOf("kr"), // default null
                        usingOsFilter = PickerUsingOsFilter.ALL, // default PickerUsingOsFilter.ALL
                        enableIndex = true, // default true
                        showMyProfile = true, // default true
                        showFavorite = true, // default true
                        showPickedFriend = true // default true
                    )
                    val chatParams = PickerTabRequestParams.ChatRequestParams(
                        selectionType = PickerChatSelectionType.CHAT, // PickerChatSelectionType.CHAT 또는 PickerChatSelectionType.CHAT_MEMBER 선택 (필수)
                        chatFilters = listOf(PickerChatFilter.REGULAR) // default listOf(PickerChatFilter.REGULAR)
                    )
                    val pickerTabRequestParams = PickerTabRequestParams(
                        title = "샘플앱 탭 피커", // default "친구 선택"
                        viewAppearance = ViewAppearance.AUTO, // default ViewAppearance.AUTO
                        orientation = PickerOrientation.AUTO, // default PickerOrientation.AUTO
                        enableSearch = true, // default true
                        disableSelectOptions = listOf(DisableSelectOption(DisableSelectReason.MSG_BLOCKED)), // default null
                        displayAllProfile = null, // default false
                        maxPickableCount = null, // default 30
                        minPickableCount = null, // default 1
                        friendsParams = friendParams, // 필수
                        chatParams = chatParams // 필수
                    )

                    PickerClient.rx.select(requireContext(), pickerTabRequestParams)
                        .subscribe({ (selectedUsers, selectedChat) ->
                            if (selectedUsers != null) {
                                Log.d(TAG, "친구 선택 성공 $selectedUsers")
                                showSnackbar("친구 선택 성공 $selectedUsers")
                            } else {
                                Log.d(TAG, "채팅방 선택 성공 $selectedChat")
                                showSnackbar("채팅방 선택 성공 $selectedChat")
                            }
                        }, { error ->
                            Log.e(TAG, "친구 또는 채팅방 선택 실패", error)
                            showSnackbar("친구 또는 채팅방 선택 실패 $error")
                        }).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("selectPopup - CHAT") {
                    val friendParams = PickerTabRequestParams.FriendsRequestParams(
                        friendFilter = PickerFriendFilter.NONE, // default PickerFriendFilter.NONE
                        countryCodeFilters = listOf("kr"), // default null
                        usingOsFilter = PickerUsingOsFilter.ALL, // default PickerUsingOsFilter.ALL
                        enableIndex = true, // default true
                        showMyProfile = true, // default true
                        showFavorite = true, // default true
                        showPickedFriend = true // default true
                    )
                    val chatParams = PickerTabRequestParams.ChatRequestParams(
                        selectionType = PickerChatSelectionType.CHAT, // PickerChatSelectionType.CHAT 또는 PickerChatSelectionType.CHAT_MEMBER 선택 (필수)
                        chatFilters = listOf(PickerChatFilter.REGULAR) // default listOf(PickerChatFilter.REGULAR)
                    )
                    val pickerTabRequestParams = PickerTabRequestParams(
                        title = "샘플앱 탭 피커", // default "친구 선택"
                        viewAppearance = ViewAppearance.AUTO, // default ViewAppearance.AUTO
                        orientation = PickerOrientation.AUTO, // default PickerOrientation.AUTO
                        enableSearch = true, // default true
                        disableSelectOptions = listOf(DisableSelectOption(DisableSelectReason.MSG_BLOCKED)), // default null
                        displayAllProfile = null, // default false
                        maxPickableCount = null, // default 30
                        minPickableCount = null, // default 1
                        friendsParams = friendParams, // 필수
                        chatParams = chatParams // 필수
                    )

                    PickerClient.rx.selectPopup(requireContext(), pickerTabRequestParams)
                        .subscribe({ (selectedUsers, selectedChat) ->
                            if (selectedUsers != null) {
                                Log.d(TAG, "친구 선택 성공 $selectedUsers")
                                showSnackbar("친구 선택 성공 $selectedUsers")
                            } else {
                                Log.d(TAG, "채팅방 선택 성공 $selectedChat")
                                showSnackbar("채팅방 선택 성공 $selectedChat")
                            }
                        }, { error ->
                            Log.e(TAG, "친구 또는 채팅방 선택 실패", error)
                            showSnackbar("친구 또는 채팅방 선택 실패 $error")
                        }).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("select - CHAT_MEMBER") {
                    val friendParams = PickerTabRequestParams.FriendsRequestParams(
                        friendFilter = PickerFriendFilter.NONE, // default PickerFriendFilter.NONE
                        countryCodeFilters = listOf("kr"), // default null
                        usingOsFilter = PickerUsingOsFilter.ALL, // default PickerUsingOsFilter.ALL
                        enableIndex = true, // default true
                        showMyProfile = true, // default true
                        showFavorite = true, // default true
                        showPickedFriend = true // default true
                    )
                    val chatParams = PickerTabRequestParams.ChatRequestParams(
                        selectionType = PickerChatSelectionType.CHAT_MEMBER, // PickerChatSelectionType.CHAT 또는 PickerChatSelectionType.CHAT_MEMBER 선택 (필수)
                        chatFilters = listOf(PickerChatFilter.REGULAR) // default listOf(PickerChatFilter.REGULAR)
                    )
                    val pickerTabRequestParams = PickerTabRequestParams(
                        title = "샘플앱 탭 피커", // default "친구 선택"
                        viewAppearance = ViewAppearance.AUTO, // default ViewAppearance.AUTO
                        orientation = PickerOrientation.AUTO, // default PickerOrientation.AUTO
                        enableSearch = true, // default true
                        disableSelectOptions = listOf(DisableSelectOption(DisableSelectReason.MSG_BLOCKED)), // default null
                        displayAllProfile = null, // default false
                        maxPickableCount = null, // default 30
                        minPickableCount = null, // default 1
                        friendsParams = friendParams, // 필수
                        chatParams = chatParams // 필수
                    )

                    PickerClient.rx.select(requireContext(), pickerTabRequestParams)
                        .subscribe({ (selectedUsers, _) ->
                            Log.d(TAG, "친구 또는 채팅방 멤버 선택 성공 $selectedUsers")
                            showSnackbar("친구 또는 채팅방 멤버 선택 성공 $selectedUsers")
                        }, { error ->
                            Log.e(TAG, "친구 또는 채팅방 멤버 선택 실패", error)
                            showSnackbar("친구 또는 채팅방 멤버 선택 실패 $error")
                        }).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("selectPopup - CHAT_MEMBER") {
                    val friendParams = PickerTabRequestParams.FriendsRequestParams(
                        friendFilter = PickerFriendFilter.NONE, // default PickerFriendFilter.NONE
                        countryCodeFilters = listOf("kr"), // default null
                        usingOsFilter = PickerUsingOsFilter.ALL, // default PickerUsingOsFilter.ALL
                        enableIndex = true, // default true
                        showMyProfile = true, // default true
                        showFavorite = true, // default true
                        showPickedFriend = true // default true
                    )
                    val chatParams = PickerTabRequestParams.ChatRequestParams(
                        selectionType = PickerChatSelectionType.CHAT_MEMBER, // PickerChatSelectionType.CHAT 또는 PickerChatSelectionType.CHAT_MEMBER 선택 (필수)
                        chatFilters = listOf(PickerChatFilter.REGULAR) // default listOf(PickerChatFilter.REGULAR)
                    )
                    val pickerTabRequestParams = PickerTabRequestParams(
                        title = "샘플앱 탭 피커", // default "친구 선택"
                        viewAppearance = ViewAppearance.AUTO, // default ViewAppearance.AUTO
                        orientation = PickerOrientation.AUTO, // default PickerOrientation.AUTO
                        enableSearch = true, // default true
                        disableSelectOptions = listOf(DisableSelectOption(DisableSelectReason.MSG_BLOCKED)), // default null
                        displayAllProfile = null, // default false
                        maxPickableCount = null, // default 30
                        minPickableCount = null, // default 1
                        friendsParams = friendParams, // 필수
                        chatParams = chatParams // 필수
                    )

                    PickerClient.rx.selectPopup(requireContext(), pickerTabRequestParams)
                        .subscribe({ (selectedUsers, _) ->
                            Log.d(TAG, "친구 또는 채팅방 멤버 선택 성공 $selectedUsers")
                            showSnackbar("친구 또는 채팅방 멤버 선택 성공 $selectedUsers")
                        }, { error ->
                            Log.e(TAG, "친구 또는 채팅방 멤버 선택 실패", error)
                            showSnackbar("친구 또는 채팅방 멤버 선택 실패 $error")
                        }).addTo(disposables)
                },
                ApiAdapter.Item.Header("Talk API"),
                ApiAdapter.Item.ApiItem("friendsForPartner()") {
                    TalkApiClient.rx.friendsForPartner()
                        .retryWhen(
                            RxAuthOperations.instance.incrementalAuthorizationRequired(
                                requireContext()
                            )
                        )
                        .subscribeOn(Schedulers.io()).subscribe(onSuccess, onError)
                        .addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("chatList()") {
                    TalkApiClient.rx.chatList()
                        .retryWhen(
                            RxAuthOperations.instance.incrementalAuthorizationRequired(
                                requireContext()
                            )
                        )
                        .subscribeOn(Schedulers.io()).subscribe(onSuccess, onError)
                        .addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("chatMembers()") {
                    TalkApiClient.rx.chatList()
                        .flatMap { chats ->
                            if (chats.elements?.isNotEmpty() == true) {
                                TalkApiClient.rx.chatMembers(chats.elements!![0].id)
                            } else {
                                Single.error<KakaoSdkError>(
                                    ClientError(
                                        ClientErrorCause.IllegalState,
                                        "No chats"
                                    )
                                )
                            }
                        }.retryWhen(
                            RxAuthOperations.instance.incrementalAuthorizationRequired(
                                requireContext()
                            )
                        )
                        .subscribeOn(Schedulers.io()).subscribe(onSuccess, onError)
                        .addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("sendCustomMessageForPartner()") {
                    TalkApiClient.rx.friendsForPartner()
                        .flatMap { friends ->
                            friends.elements?.let { friend ->
                                FriendsActivity.startForResult(
                                    requireContext(),
                                    friend.map(PickerItem.Companion::fromPartnerFriend)
                                )
                            } ?: Single.just(listOf())
                        }
                        .observeOn(Schedulers.io())
                        .flatMap {
                            TalkApiClient.rx.sendCustomMessageForPartner(
                                it,
                                templateIds.getLong("customMessage")
                            )
                        }
                        .retryWhen(
                            RxAuthOperations.instance.incrementalAuthorizationRequired(
                                requireContext()
                            )
                        )
                        .subscribeOn(Schedulers.io())
                        .subscribe(onSuccess, onError).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("sendScrapMessageForPartner()") {
                    TalkApiClient.rx.friendsForPartner()
                        .flatMap { friends ->
                            friends.elements?.let {
                                FriendsActivity.startForResult(
                                    requireContext(),
                                    it.map(PickerItem.Companion::fromPartnerFriend)
                                )
                            } ?: Single.just(listOf())
                        }
                        .observeOn(Schedulers.io())
                        .flatMap {
                            TalkApiClient.rx.sendScrapMessageForPartner(
                                it,
                                "https://developers.kakao.com"
                            )
                        }
                        .subscribeOn(Schedulers.io()).subscribe(onSuccess, onError)
                        .addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("sendDefaultMessageForPartner()") {
                    TalkApiClient.rx.friendsForPartner().flatMap { friends ->
                        friends.elements?.let {
                            FriendsActivity.startForResult(
                                requireContext(),
                                it.map(PickerItem.Companion::fromPartnerFriend)
                            )
                        } ?: Single.just(listOf())
                    }.flatMap {
                        TalkApiClient.rx.sendDefaultMessageForPartner(
                            it,
                            OpenFragmentRx.defaultLocation
                        )
                    }.retryWhen(
                        RxAuthOperations.instance.incrementalAuthorizationRequired(
                            requireContext()
                        )
                    )
                        .subscribeOn(Schedulers.io()).subscribe(onSuccess, onError)
                        .addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("sendCustomMessageToChats()") {
                    TalkApiClient.rx.chatList()
                        .flatMap { friends ->
                            friends.elements?.let { friend ->
                                FriendsActivity.startForResult(
                                    requireContext(),
                                    friend.map(PickerItem.Companion::fromChat)
                                )
                            } ?: Single.just(listOf())
                        }
                        .observeOn(Schedulers.io())
                        .flatMap { chatId ->
                            TalkApiClient.rx.sendCustomMessageToChats(
                                chatId.map { it.toLong() },
                                templateIds.getLong("customMessage")
                            )
                        }
                        .retryWhen(
                            RxAuthOperations.instance.incrementalAuthorizationRequired(
                                requireContext()
                            )
                        )
                        .subscribeOn(Schedulers.io()).subscribe(onSuccess, onError)
                        .addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("sendDefaultMessageToChats()") {
                    TalkApiClient.rx.chatList()
                        .flatMap { friends ->
                            friends.elements?.let {
                                FriendsActivity.startForResult(
                                    requireContext(),
                                    it.map(PickerItem.Companion::fromChat)
                                )
                            } ?: Single.just(listOf())
                        }.map { chatId -> chatId.map { it.toLong() } }
                        .observeOn(Schedulers.io())
                        .flatMap {
                            TalkApiClient.rx.sendDefaultMessageToChats(
                                it,
                                OpenFragmentRx.defaultFeed
                            )
                        }
                        .subscribeOn(Schedulers.io())
                        .subscribe(onSuccess, onError).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("sendScrapMessageToChats()") {
                    TalkApiClient.rx.chatList()
                        .flatMap { friends ->
                            friends.elements?.let {
                                FriendsActivity.startForResult(
                                    requireContext(),
                                    it.map(PickerItem.Companion::fromChat)
                                )
                            } ?: Single.just(listOf())
                        }
                        .map { chatId -> chatId.map { it.toLong() } }
                        .observeOn(Schedulers.io())
                        .flatMap {
                            TalkApiClient.rx.sendScrapMessageToChats(
                                it,
                                "https://developers.kakao.com"
                            )

                        }
                        .subscribeOn(Schedulers.io()).subscribe(onSuccess, onError)
                        .addTo(disposables)
                })
        )

        view.findViewById<RecyclerView>(R.id.recyclerView).apply {
            setHasFixedSize(true)
            addItemDecoration(
                DividerItemDecoration(context, DividerItemDecoration.VERTICAL)
            )
            layoutManager = viewManager
            adapter = viewAdapter
        }
    }

    private lateinit var viewAdapter: RecyclerView.Adapter<*>
    private lateinit var viewManager: RecyclerView.LayoutManager
    private val disposables = CompositeDisposable()

    private lateinit var templateIds: Bundle


    companion object {
        const val TAG = "KakaoSDKSample"

        fun newInstance(templateIds: Bundle): PartnerFragmentRx =
            PartnerFragmentRx().apply {
                arguments = templateIds
            }
    }
}
